/**
 * @file toggle.c
 *
 * This program demonstrates starting COG C code from C++
 * The cog makes all IO except 30/31 toggle.
 *
 * The demo includes libtiny.a for small C++ footprint in LMM mode.
 * libtiny.a should not be included when using XMMC or other XMM modes.
 *
 * Copyright (c) 2012 by John Steven Denson
 * All rights MIT licensed.
 * See license statement at the end of this file for specific terms.
 */
 
#include <stdio.h>
#include "propeller.h"
#include "toggle.h"

extern _Driver _FullDuplexSerialDriver;

/* This is a list of all drivers we can use in the
 * program. The default _InitIO function opens stdin,
 * stdout, and stderr based on the first driver in
 * the list (the serial driver, for us).
 * This can be removed so that the much smaller
 * SimpleSerial driver can be use.
 */
_Driver *_driverlist[] = {
  &_FullDuplexSerialDriver,
  NULL
};

int main (void)
{
    extern unsigned int _C_LOCK;
    _C_LOCK |= 256;
    
    /* Toggle class controls LEDs on Quickstart pins.
     * Toggle lives in the stack. No new/delete needed.
     */
    Toggle toggle(0x7f << 16);

    /* pause for terminal startup
     */
    waitcnt(CLKFREQ/2+CNT);
    
    /* start the new cog
     */
    int cog = toggle.start();
    printf("\nToggle COG has started %d\n", cog);

    /* The main while thread toggles a separate led
     */
    int led  = 0x80 << 16;
    DIRA |= led;

    while(1) {
    
        /* do update once a second */
        waitcnt(CLKFREQ+CNT);

        /* set our LED */
        OUTA ^= led;
        /* update the .cogc toggle parameters */
        toggle.update();
        
        /* read and display the toggle parameters */
        printf("Toggle count = %d, wait = %d\n",
            toggle.getCount(), toggle.getWait());
    }
    
    /* program should get here, but return 0 anyway for main */
    return 0;
}


/* +--------------------------------------------------------------------
 * | TERMS OF USE: MIT License
 * +--------------------------------------------------------------------
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files
 * (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 * +--------------------------------------------------------------------
 */
